/* main.cpp
 *
 * $Id: main.cpp,v 1.11 2000/12/10 11:03:06 amos Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <qapplication.h>
#include <qmessagebox.h>

#include <signal.h>

#include "qmmainwindow.h"

/*! \file main.cpp
  Implementation of the main function and other startup functions.
*/

/*!
  Handler for seg. faults etc.  Just show a friendly dialog that we
  are going to die now :-)
*/

void sighandler(int sig)
{
	signal(SIGILL, 0);
	signal(SIGSEGV, 0);
	signal(SIGPIPE, 0);
	
	QMessageBox::critical(
		0, QObject::tr("Apollo Fatal Error"),
		QObject::tr("<h1>Fatal Error</h1>"
					"An internal error happened within Apollo. The signal received was: "
					"<p>"
					"<center>%1</center>"
					"<p>"
					"If you can reproduce this error, please report the bug to us "
					"and give a detailed description on how to reproduce the bug. "
					"You can report the bug in two ways:"
					"<ul>"
					"<li>http://bugs.apolloplayer.org</li>"
					"<li>bugs@apolloplayer.org</li>"
					"</ul>"
					"Thank you.<br>"
					"<p>"
					"Apollo will need to be restarted.").arg(sig),
		QObject::tr("&Quit"));
}

/*! \file main.cpp
  Where the magic starts... :-)
*/

/*!
  \mainpage Apollo

  \section intro Introduction

  This is the class documentation for <a href="http://www.apolloplayer.org">Apollo</a>,
  a sound player for Linux. It was generated with <a href="http://www.doxygen.org">Doxygen</a>.

  \section compilation Compilation

  Included in the source directory, there should be a Makefile, so simply typing

  \code
  make
  \endcode

  at the command prompt should build the 'apollo' executable.  Please see README
  and INSTALL for more information.

  \section running Running

  Typing

  \code
  ./apollo
  \endcode

  runs Apollo.

  \section license License

  Apollo is distributed under the <a href="http://www.gnu.org/copyleft/gpl.html">GPL license</a>.
 */

int main(int argc, char **argv)
{

    signal(SIGILL, sighandler);
    signal(SIGSEGV, sighandler);
    signal(SIGPIPE, sighandler);	
	
    QApplication a(argc, argv);

    QmMainWindow *mainwin = new QmMainWindow(0, QObject::tr("Apollo Main Window"));
    mainwin->show();
    mainwin->loadStartupPlayList();
    QObject::connect(qApp, SIGNAL(lastWindowClosed()),
					 qApp, SLOT(quit()));
    int rc = a.exec();
    mainwin->saveStartupPlayList();

    delete mainwin;
    
    return rc;
}
