/* qmdefaultmenuitem.cpp
 *
 * $Id: qmdefaultmenuitem.cpp,v 1.2 2000/12/09 12:12:36 amos Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmdefaultmenuitem.h"

#include <qfont.h>
#include <qpainter.h>
#include <qfontmetrics.h>

/*! \file qmdefaultmenuitem.cpp
  Implementation of QmDefaultMenuItem class which represents the default menu item.
*/

/*!
  \class QmDefaultMenuItem qmdefaultmenuitem.h
  \brief Represents the default menu item.

  This class provides a menu item that draws its label in bold.  This is the
  only difference between this class and the standard menu items.
  Instances of this class can be used to indicate default menu items.

  \note It is up to the programmer to make sure that there is only one
  default item in the menu.
*/


/*!
  Constructs a menu item with label \a label and font \a font.
  There is no need to pass a bold font as it will be made bold.
*/

QmDefaultMenuItem::QmDefaultMenuItem(
	const QString &label,
	const QFont &font)
	: m_Label( label ),
	  m_Font( font )
{
	m_Font.setBold(true);
}


/*!

*/

QmDefaultMenuItem::~QmDefaultMenuItem()
{
}


/*!
  Draws the menu label in bold.
 */

void
QmDefaultMenuItem::paint(
	QPainter *p,
	const QColorGroup &,
	bool,
	bool,
	int x,
	int y,
	int w,
	int h)
{
	p->setFont( m_Font );
	p->drawText( x, y, w, h, AlignLeft | AlignVCenter | ShowPrefix | DontClip, m_Label );
}


/*!

 */

QSize
QmDefaultMenuItem::sizeHint()
{
	return QFontMetrics( m_Font ).size( AlignLeft | AlignVCenter | ShowPrefix | DontClip, m_Label );
}

