/* qmmainwindow.cpp
 *
 * $Id: qmmainwindow.cpp,v 1.74 2000/12/18 17:58:52 amos Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include <fcntl.h>
#include <iostream.h>
#include <linux/soundcard.h>
#include <math.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <unistd.h>

#include <qaction.h>
#include <qdialog.h>
#include <qapplication.h>
#include <qframe.h>
#include <qheader.h>
#include <qiconset.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qmainwindow.h>
#include <qmessagebox.h>
#include <qobject.h>
#include <qpixmap.h>
#include <qpopupmenu.h>
#include <qslider.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qtextview.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <qwidget.h>
#include <qinputdialog.h>

#include "qmaboutdialog.h"
#include "qmconfig.h"
#include "qmdefaultmenuitem.h"
#include "qmdiritem.h"
#include "qmmainwindow.h"
#include "qmmpg123player.h"
#include "qmplaylist.h"
#include "qmsongitem.h"
#include "version.h"
//#include "qmconfigdialog.h"

#include "icons/eject.xpm"
//  #include "icons/minimize.xpm"
#include "icons/next.xpm"
#include "icons/options.xpm"
#include "icons/pause.xpm"
#include "icons/play.xpm"
#include "icons/playlist.xpm"
#include "icons/prev.xpm"
#include "icons/stop.xpm"
#include "icons/songitem.xpm"
#include "icons/folder.xpm"
#include "icons/apollo-mini.xpm"

/*! \file qmmainwindow.cpp
  Implementation of QmMainWindow class which is the main Apollo window.
*/

/*!
  \class QmMainWindow qmmainwindow.h
  \brief The main Apollo window.

  This is the main window of Apollo and contains the player controls
  as well as the play list.  This class is central in Apollo.  It's
  from here the user start and spends most of the time.

  \todo Move the volume related functions into a single class inheriting from QSlider
*/


/*!
  Constructs a new Apollo main window.
*/

QmMainWindow::QmMainWindow(
    QWidget *parent,
    const char *name)
    : QMainWindow(parent, name),
    m_State( QmMainWindow::Ready ),
    m_PlayState( QmMainWindow::Stopped ),
    m_ShortMessageTime( 2000 ),
    m_FileDialog( 0, "file-dialog", true )
{
    QmConfig *conf = QmConfig::instance();
    conf->load();
    m_MusicDir = conf->currentMusicPath();

	m_StopClicked = false;
    m_VolumePressed = false;
    m_ShowNameStatus = true;

    setIcon( QPixmap( apollo_32x32_xpm ) );

    m_FileDialog.setDir( QDir( m_MusicDir ) );

    setCaption("Apollo");
    resize( conf->windowSize() );
    move( conf->windowPosition() );

    if ( conf->fileDialogSize().isValid() )
    {
        m_FileDialog.resize( conf->fileDialogSize() );
        m_FileDialog.move( conf->fileDialogPosition() );
    }

    QToolBar *playbar = new QToolBar(this);
	playbar->setHorizontalStretchable(true);
	addToolBar(playbar);
	
    m_pFileMenu = new QPopupMenu(this);
    m_pFileMenu->insertItem(QIconSet( QPixmap (songitem_xpm)),
							tr("Add songs..."),
							this,
							SLOT(addFiles()),
							CTRL + Key_F);
	QFont f = m_pFileMenu->font();
	int adddirid = m_pFileMenu->insertItem(new QmDefaultMenuItem(tr("Add dir..."), f));
	m_pFileMenu->setAccel(CTRL + Key_D, adddirid);
	m_pFileMenu->connectItem(adddirid, this, SLOT(addDir()));
    m_pFileMenu->changeItem( adddirid, QIconSet( QPixmap(folder_xpm) ), QPixmap(folder_xpm) );

	/*
	QFont f = m_pFileMenu->font();
	int removeid = m_pFileMenu->insertItem(new QmDefaultMenuItem(tr("Remove"), f));
	m_pFileMenu->setAccel(Key_Delete, removeid);
	m_pFileMenu->connectItem(removeid, this, SLOT(removeItem()));
	*/
	
    m_pFileMenu->insertItem(tr("Open playlist..."),
							this,
							SLOT(loadPlayList()),
							CTRL + Key_O);
    m_pFileMenu->insertItem(tr("Save playlist..."),
							this,
							SLOT(savePlayList()),
							CTRL + Key_S);
    m_pFileMenu->insertSeparator();
    m_pFileMenu->insertItem(tr("Quit"),
							qApp,
							SLOT(quit()),
							CTRL + Key_Q);

    m_pAddFiles = new QmMenuButton(QIconSet( QPixmap(eject_xpm) ),
								   tr("Add files"),
								   playbar);
	connect(m_pAddFiles, SIGNAL(menuClicked()), this, SLOT(popupFileMenu()));
	connect(m_pAddFiles, SIGNAL(iconClicked()), this, SLOT(addDir()));
	
    m_pPlaylistMenu = new QPopupMenu(this);
    m_pPlaylistMenu->insertItem(tr("Clear"), this, SLOT(clearPlayList()));
	
	f = m_pPlaylistMenu->font();
	int removeid = m_pPlaylistMenu->insertItem(new QmDefaultMenuItem(tr("Remove"), f));
	m_pPlaylistMenu->setAccel(Key_Delete, removeid);
	m_pPlaylistMenu->connectItem(removeid, this, SLOT(removeItem()));
	
    m_pPlaylistMenu->insertItem(tr("Mark as bad"), this, SLOT(setMark()));
    m_pPlaylistMenu->insertItem(tr("Clear bad mark"), this, SLOT(clearMark()));

    m_pPlaylistButton = new QmMenuButton(QIconSet( QPixmap(playlist_xpm) ),
										 tr("Modify playlist"),
										 playbar);
	connect(m_pPlaylistButton, SIGNAL(menuClicked()), this, SLOT(popupPlaylistMenu()));
	connect(m_pPlaylistButton, SIGNAL(iconClicked()), this, SLOT(removeItem()));
	
    m_pOptionsMenu = new QPopupMenu(this);
	f = m_pOptionsMenu->font();
	int settingsid = m_pOptionsMenu->insertItem(new QmDefaultMenuItem(tr("Settings..."), f));
	m_pOptionsMenu->connectItem(settingsid, this, SLOT(openConfigDialog()));
//    m_pOptionsMenu->insertItem(tr("Settings..."), this, SLOT(openConfigDialog()));
	m_pOptionsMenu->insertSeparator();
    m_pOptionsMenu->insertItem(tr("Respawn mpg123 player"), this, SLOT(startMpg123()));
	m_pOptionsMenu->insertSeparator();
    m_pOptionsMenu->insertItem(tr("About Qt..."), this, SLOT(aboutQt()));
    m_pOptionsMenu->insertItem(tr("About..."), this, SLOT(about()));

    m_pOptions = new QmMenuButton(QIconSet( QPixmap(options_xpm) ),
								  tr("Options"),
								  playbar);
	connect(m_pOptions, SIGNAL(menuClicked()), this, SLOT(popupOptionsMenu()));
	connect(m_pOptions, SIGNAL(iconClicked()), this, SLOT(openConfigDialog()));
	
	playbar->addSeparator();
	
    m_pPrev = new QToolButton(QIconSet( QPixmap(prev_xpm) ),
							  tr("Previous (Shortcut: 1)"),
							  tr("Play previous song"),
							  this,
							  SLOT(prev()),
							  playbar);
	m_pPrev->setAccel('1');
    
    m_pStop = new QToolButton(QIconSet( QPixmap(stop_xpm) ),
							  tr("Stop (Shortcut: 2)"),
							  tr("Stop the playing song"),
							  this,
							  SLOT(stop()),
							  playbar);
	m_pStop->setAccel('2');

    m_pPause = new QToolButton(QIconSet( QPixmap(pause_xpm) ),
							   tr("Pause (Shortcut: 3)"),
							   tr("Pause the playing song"),
							   this,
							   SLOT(pause()),
							   playbar);
	m_pPause->setToggleButton(true);
	m_pPause->setAccel('3');

    m_pPlay = new QToolButton(QIconSet( QPixmap(play_xpm) ),
							  tr("Play (Shortcut: 4)"),
							  tr("Play the highlighted song"),
							  this,
							  SLOT(play()),
							  playbar);
	m_pPlay->setAccel('4');

    m_pNext = new QToolButton(QIconSet( QPixmap(next_xpm) ),
							  tr("Next (Shortcut: 5)"),
							  tr("Play next song"),
							  this,
							  SLOT(next()),
							  playbar);
	m_pNext->setAccel('5');

	playbar->addSeparator();
	
	/*
    m_pMinimize = new QToolButton(QIconSet( QPixmap(minimize_xpm) ),
								  tr("Minimize (Short cut: 8)"),
								  tr("Minimize window"),
								  this,
								  SLOT(togglePlaylist()),
								  playbar);
	m_pMinimize->setToggleButton(true);
	m_pMinimize->setAccel('8');
	*/

	QLabel *stretch =  new QLabel("", playbar);
	stretch->setBackgroundColor(playbar->backgroundColor());
	playbar->setStretchableWidget(stretch);
	
    QToolBar *statusbar = new QToolBar(this);
	statusbar->setHorizontalStretchable(true);
	addToolBar(statusbar, Top, true);
    m_pStatus = new QLabel(statusbar);
    m_pStatus->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	m_pStatus->setFixedHeight(m_pStatus->sizeHint().height());
	m_pStatus->setBackgroundColor(white);

    m_pWidget = new QWidget();

    m_pPlayList = new QmPlayList(this);
    m_pPlayList->addColumn(tr("Name"), 300);
    m_pPlayList->addColumn(tr("Length"));
    m_pPlayList->setAllColumnsShowFocus(true);
	m_pPlayList->setRootIsDecorated(false);
    m_pPlayList->setItemMargin(3);

//    m_pPlayList->setColumnWidthMode(0, QListView::Maximum);

	setCentralWidget(m_pPlayList);

    connect(m_pPlayList, SIGNAL(doubleClicked(QListViewItem*)),
			this, SLOT(selected(QListViewItem*)));
    connect(m_pPlayList, SIGNAL(returnPressed(QListViewItem*)),
			this, SLOT(selected(QListViewItem*)));

    QToolBar *positionbar = new QToolBar(this);
	positionbar->setHorizontalStretchable(true);
	addToolBar(positionbar, Bottom);	
	(void) new QLabel(tr("Position"), positionbar);
	m_pTracker = new QSlider(0, 0, 1, 0, Horizontal, positionbar);

    QToolBar *volumebar = new QToolBar(this);
	volumebar->setHorizontalStretchable(true);
	addToolBar(volumebar, Bottom);	
	(void) new QLabel(tr("Volume"), volumebar);
	m_pVolume = new QSlider(0, 100, 1, 0, Horizontal, volumebar);
    m_pVolume->setValue( m_Mixer.volume() );

	connect(m_pVolume, SIGNAL(valueChanged(int)), this, SLOT(volumeChanged(int)));
	connect(m_pVolume, SIGNAL(sliderPressed()), this, SLOT(volumePressed()));
	connect(m_pVolume, SIGNAL(sliderReleased()), this, SLOT(volumeReleased()));

    m_pVolumeTimer = new QTimer( this );
    connect( m_pVolumeTimer, SIGNAL( timeout() ),
             this, SLOT( refreshVolume() ) );
    m_pVolumeTimer->start( 100, false );

    m_pStatusTimer = new QTimer( this );
    connect( m_pStatusTimer, SIGNAL( timeout() ),
             this, SLOT( resetStatus() ) );

	startMpg123();

	QObject::connect(m_pPlayer, SIGNAL(playingStopped()), this, SLOT(playingStopped()));

	QObject::connect(m_pPlayer, SIGNAL(secondsCount(int)), this, SLOT(secondsCount(int)));
	QObject::connect(m_pPlayer, SIGNAL(secondsPlaying(int)), this, SLOT(secondsPlaying(int)));
	QObject::connect(m_pPlayer, SIGNAL(frameCount(int)), this, SLOT(setTrackerEnd(int)));
	QObject::connect(m_pPlayer, SIGNAL(framesLeft(int)), m_pTracker, SLOT(setValue(int)));
    QObject::connect(m_pPlayer, SIGNAL(songTitle(const QString &)),
                     this, SLOT( setSongTitle(const QString &)));

	
    updateVolume( false );
    updateWidgets();
    updateControls();
    updatePositionControls();

    showMessage( tr( "Welcome to Apollo!" ) );
}


/*!
  Destructs the main window.  Window position, size etc. will be
  saved.
*/

QmMainWindow::~QmMainWindow()
{
	delete m_pPlayer;

    QmConfig *conf = QmConfig::instance();

    conf->setWindowSize( size() );
    conf->setWindowPosition( pos() );

    conf->setFileDialogSize( m_FileDialog.size() );
    conf->setFileDialogPosition( m_FileDialog.pos() );

    conf->setScrollPosition( QPoint( m_pPlayList->contentsX(), m_pPlayList->contentsY() ) );

    conf->setCurrentMusicPath( m_MusicDir );
	
    conf->save();
}


/*!
  Loads the current playlist in an internal file for playlist persistence
  and restores the current playing song from the config.
*/

void
QmMainWindow::loadStartupPlayList()
{
    setState( Busy );
    // Restore playlist

//      QFileInfo playlist( "playlist.m3u" );
    QFileInfo playlist_home( QDir::homeDirPath() + "/.apollo/playlist.m3u" );
//      if ( playlist.exists() )
//      {
//          m_pPlayList->load( playlist.filePath() );
//          m_PlayListName = playlist.filePath();
//      }
//      else
    if ( playlist_home.exists() )
    {
        m_pPlayList->load( playlist_home.filePath() );
    }
    m_PlayListName = playlist_home.filePath();

    QmConfig *conf = QmConfig::instance();

    // Restore last playing song
	QListViewItem *lvitem = m_pPlayList->firstChild();
	QmSongItem *songitem;

    if ( m_pPlayList->playingSong() == 0 )
    {
        while(lvitem != 0 )
        {
            songitem = dynamic_cast<QmSongItem*>(lvitem);
            if(songitem != 0)
            {
                if(songitem->filePath() == conf->playingSong())
                {
                    songitem->setPlaying(true);
                    m_pPlayList->setPlaying(songitem);
                    m_pPlayList->setSelected(songitem, true);
//                      m_pPlayList->setCurrentItem(songitem);
                    break;
                }
            }
            lvitem = lvitem->itemBelow();
        }
    }

    m_pPlayList->setContentsPos( conf->scrollPosition().x(), conf->scrollPosition().y() );
    setState( Ready );
    updateControls();
    updatePositionControls();
}


/*!
  Stores the current playlist in an internal file for playlist persistence
  and stores the current playing song in the config.
*/

void
QmMainWindow::saveStartupPlayList()
{
    setState( Busy );
    // Store playlist
    if ( !m_PlayListName.isEmpty() )
    {
        if ( !m_pPlayList->save( m_PlayListName ) )
        {
            QMessageBox::warning( this, tr( "Apollo - warning" ),
                                  tr( "Apollo was unable to save the current playlist in\n"
                                      "%1\n\n"
                                      "Please make sure that Apollo has write access to this file." )
                                  .arg( m_PlayListName ) );
        }
    }

    QmConfig *conf = QmConfig::instance();

    // Store current song being played, if any
	if(m_pPlayList->playingSong())
		conf->setPlayingSong(m_pPlayList->playingSong()->filePath());
	else
		conf->setPlayingSong("");
    setState( Ready );
}

/*!
  Shows the file menu.
 */

void
QmMainWindow::popupFileMenu()
{
    m_pFileMenu->popup(mapToGlobal(m_pAddFiles->geometry().bottomLeft()));
	m_pVolume->setValue(m_Mixer.volume());
}


/*!
  Shows the playlist menu.
 */

void
QmMainWindow::popupPlaylistMenu()
{
    m_pPlaylistMenu->popup(mapToGlobal(m_pPlaylistButton->geometry().bottomLeft()));
}


/*!
  Shows the options menu.
 */

void
QmMainWindow::popupOptionsMenu()
{
    m_pOptionsMenu->popup(mapToGlobal(m_pOptions->geometry().bottomLeft()));
}


/*!
  Adds one or more songs to the list.
 */

void
QmMainWindow::addFiles()
{
    if ( state() == QmMainWindow::Busy ) // Do not show dialogs when busy
        return;

    m_FileDialog.setMode( QFileDialog::ExistingFiles );
    m_FileDialog.setFilter( "MP3 files (*.mp3)" );
    m_FileDialog.setDir( QDir( m_MusicDir ) );
    m_FileDialog.setCaption( "Apollo - Add mp3 files" );
    int ret = m_FileDialog.exec();
    if ( ret == QDialog::Accepted )
    {
        m_MusicDir = m_FileDialog.dirPath();
        QStringList strlist = m_FileDialog.selectedFiles();

        QmSongItem *first_item = 0;

        QStringList::ConstIterator it;
        for(it = strlist.begin(); it != strlist.end(); ++it)
        {
            QmSongItem *item = new QmSongItem(m_pPlayList, *it);
            m_pPlayList->ensureItemVisible( item );
            if ( first_item == 0 )
                first_item = item;
        }
        if ( first_item )
            m_pPlayList->ensureItemVisible( first_item );

        int w = m_pPlayList->header()->width();
        w = w > 400 ? w : 400;
        setMaximumSize(w, maximumSize().height());

        updateControls();
        updatePositionControls();
    }
}


/*!
  Adds a directory to the play list.  The directory will be recursively
  read, and each directory containing a supported sound file will be added
  together with the sound file(s) the directory contain.
 */

void
QmMainWindow::addDir()
{
    if ( state() == QmMainWindow::Busy ) // Do not show dialogs when busy
        return;

//    QString dirname = QFileDialog::getExistingDirectory();
    m_FileDialog.setDir( QDir( m_MusicDir ) );
    m_FileDialog.setMode( QFileDialog::Directory );
    m_FileDialog.setFilter( "MP3 files (*.mp3)" );
    m_FileDialog.setCaption( "Apollo - Add directory" );
    int ret = m_FileDialog.exec();

    QmDirItem *first_item = 0;

    if ( ret == QDialog::Accepted )
    {
        QString dirname = m_FileDialog.dirPath();
        m_MusicDir = dirname;

        if( ! dirname.isEmpty())
        {
            setState( Busy );
            QmDirItem *item = new QmDirItem(m_pPlayList, dirname);
            if ( first_item == 0 )
                first_item = item;
            setState( Ready );
            updateControls();
            updatePositionControls();
        }
    }

    if ( first_item )
        m_pPlayList->ensureItemVisible( first_item );
}


/*!
  Plays the selected item.  If not item is selected, the top most element will
  be played.

  \sa stop(), pause()
 */

void
QmMainWindow::play()
{
    QmSongItem *item = dynamic_cast<QmSongItem*>(m_pPlayList->selectedItem());

	if(item == 0)
		item = dynamic_cast<QmSongItem*>(m_pPlayList->firstChild());
	
    if(item != 0)
		selected(item);
}


/*!
  Stops the music.

  \sa play(), pause()
 */

void
QmMainWindow::stop()
{
	m_StopClicked = true;
	m_pPlayer->stop();
    setPlayingState( QmMainWindow::Stopped );
//  	m_pPause->setEnabled(false);
//  	m_pStop->setEnabled(false);
}


/*!
  Pauses the music.

  \sa play(), stop()
*/

void
QmMainWindow::pause()
{
	m_pPlayer->pause();
    setPlayingState( QmMainWindow::Paused );
    showSongInfo();
}


/*!
  Selectes the previous song item, if any, and plays it.  Folder items
  and bad songs will be skipped.

  \sa next()
 */

void
QmMainWindow::prev()
{
    if ( playingState() == QmMainWindow::Stopped )
    {
        QmSongItem *songitem = m_pPlayList->prevSong();
        songitem->setPlaying(true);
        m_pPlayList->setPlaying(songitem);
        m_pPlayList->setSelected(songitem, true);
        m_pPlayList->ensureItemVisible(songitem);
    }
    else
        selected(m_pPlayList->prevSong());
    updateControls();
    updatePositionControls();
    showSongInfo();
}


/*!
  Selects the next song item, if any, and plays it.  Folder items and bad
  songs will be skipped.

  \sa prev()
 */

void
QmMainWindow::next()
{
    if ( playingState() == QmMainWindow::Stopped )
    {
        QmSongItem *songitem = m_pPlayList->nextSong();
        songitem->setPlaying(true);
        m_pPlayList->setPlaying(songitem);
        m_pPlayList->setSelected(songitem, true);
        m_pPlayList->ensureItemVisible(songitem);
    }
    else
        selected(m_pPlayList->nextSong());
    updateControls();
    updatePositionControls();
    showSongInfo();
}


/*!
  Plays, if possible, \a item.  If \a item is a directory or a song marked as
  bad, this function does nothing.
 */

void
QmMainWindow::selected(
    QListViewItem *item)
{
    QmSongItem *songitem = dynamic_cast<QmSongItem*>(item);

    if(songitem == 0)
    {
        setPlayingState( QmMainWindow::Stopped );
		return;
    }

	if(songitem->isBad())
    {
        setPlayingState( QmMainWindow::Stopped );
		return;
    }

	m_pPlayList->setPlaying(songitem);

	// The reason for this variable is to distinguish by user Stop action
	// and end of song Stop.
	
	m_StopClicked = false;
	
    m_NameString 	  = QString::null;
    m_TimeString 	  = QString::null;
    m_TimeTotalString = QString::null;
    m_pPlayer->play(songitem->filePath());
    setPlayingState( QmMainWindow::Playing );
    updatePositionControls();
    showSongInfo();
}



/*!
  Hides/shows the play list.

  \todo Obviously, this one is not functional yet :-)
 */

void
QmMainWindow::togglePlaylist()
{
}


/*!
  Opens up a file requester asking for a play list.
 */

void
QmMainWindow::loadPlayList()
{
    if ( state() == QmMainWindow::Busy ) // Do not show dialogs when busy
        return;

    QString s = QFileDialog::getOpenFileName();

    setState( Busy );
    if( ! s.isEmpty())
		m_pPlayList->load(s);
    setState( Ready );
    updateControls();
    updatePositionControls();
}


/*!
  Opens up a file requester asking to save the play list.
 */

void
QmMainWindow::savePlayList()
{
    if ( state() == QmMainWindow::Busy ) // Do not show dialogs when busy
        return;

    QString s = QFileDialog::getSaveFileName();

    setState( Busy );
    if( ! s.isEmpty())
		m_pPlayList->save(s);
    setState( Ready );
}


/*!
  Shows information about Apollo.
 */

void
QmMainWindow::about()
{
    if ( state() == QmMainWindow::Busy ) // Do not show dialogs when busy
        return;

	QmAboutDialog dlg(this, "about", true);
    dlg.adjustSize();
    dlg.move( QApplication::desktop()->geometry().center() - dlg.rect().center() );
	dlg.show();

	// Try signal handler:
//	int *ptr = 0;
//	*ptr = 1;
}


/*!
  Shows information about Qt.
*/

void
QmMainWindow::aboutQt()
{
	QMessageBox::aboutQt(this, tr("Apollo - About Qt"));
}


/*!
  Called by the player whenever the second counter changes, this will update the status bar song info with the new time.
*/

void
QmMainWindow::secondsCount(
	int s)
{
	int min, sec, hr;

	sec = s % 60;
	min = s / 60;
    hr = min / 60;
    min %= 60;
	QString str("%1:%2");
	QString str_hr("%1:%2:%3");

    QString min_str;
    min_str.sprintf( "%.2d", min );
    QString sec_str;
    sec_str.sprintf( "%.2d", sec );

    if ( hr == 0 )
        m_TimeTotalString = str.arg(min_str).arg(sec_str);
    else
        m_TimeTotalString = str_hr.arg(hr).arg(min_str).arg(sec_str);
    updateStatus();

    QmSongItem *songitem = dynamic_cast<QmSongItem*>(m_pPlayList->selectedItem());

    if(songitem == 0)
		return;
    songitem->setSongLength( m_TimeTotalString );
    m_pPlayList->triggerUpdate();
}


/*!
  Updates the status bar with play info, that is, how many seconds
  that has been played.
*/

void
QmMainWindow::secondsPlaying(
	int s)
{
	int min, sec, hr;

	sec = s % 60;
	min = s / 60;
    hr = min / 60;
    min %= 60;
	QString str("%1:%2");
	QString str_hr("%1:%2:%3");

    QString min_str;
    min_str.sprintf( "%.2d", min );
    QString sec_str;
    sec_str.sprintf( "%.2d", sec );

    if ( hr == 0 )
        m_TimeString = str.arg(min_str).arg(sec_str);
    else
        m_TimeString = str_hr.arg(hr).arg(min_str).arg(sec_str);
    updateStatus();
}


/*!
  Updates status bar and window caption with the song title
  \a title.

  \sa updateStatus(), updateCaption()
  \todo Currently, this function will also update the item itself
  in the list view with its ID3 information.  This may be replaced
  with a full ID3 info "parser" later.
*/

void
QmMainWindow::setSongTitle( const QString &title )
{
    m_NameString = title;
    updateStatus();
    updateCaption();

    QmSongItem *songitem = dynamic_cast<QmSongItem*>(m_pPlayList->selectedItem());

    if(songitem == 0)
		return;
    songitem->setSongTitle( title );
    m_pPlayList->triggerUpdate();
}


/*!
  Updates the status bar.  Typically, playing info is shown, such as
  time played and time remaining, but other messages may also be shown,
  such as when adjusting the volume.
*/

void
QmMainWindow::updateStatus()
{
    if ( m_ShowNameStatus )
    {
        QString status_msg;
        if ( playingState() == QmMainWindow::Playing ||
             playingState() == QmMainWindow::Paused )
        {
            if ( m_NameString.isEmpty() ) // If empty just ignore showing
                return;

            status_msg += m_NameString;
            if ( !m_TimeString.isEmpty() && !m_TimeTotalString.isEmpty() )
            {
                status_msg += QString( " (%1/%2)" )
                    .arg( m_TimeString )
                    .arg( m_TimeTotalString );
            }
            else if ( m_TimeString.isEmpty() && m_TimeTotalString.isEmpty() )
            {
            }
            else
            {
                status_msg += " (NA)";
            }
        }
        else if ( playingState() == QmMainWindow::Stopped )
        {
        }
        else
        {
            // This should never happen
        }

        m_pStatus->setText( status_msg );
    }
    else
    {
        m_pStatus->setText( m_MessageString );
    }
}


/*!
  Updates the window caption according to the song playing.
*/

void
QmMainWindow::updateCaption()
{
    QString title( "Apollo" );
    if ( playingState() == QmMainWindow::Playing ||
         playingState() == QmMainWindow::Paused )
    {
        title += QString( " - " ) + m_NameString;
    }
    setCaption( title );
}


/*!
  Sets the end (upper) value of the position tracker to \a s.
*/

void
QmMainWindow::setTrackerEnd(
	int s)
{
	m_pTracker->setMaxValue(s);
}


/*!
  Adjust the volume to \a v, this is called whenever the volume slider changes.
*/

void
QmMainWindow::volumeChanged(
	int v)
{
    if ( !m_VolumePressed )
        m_pVolumeTimer->blockSignals( true );
	m_Mixer.setVolume(v);
    if ( !m_VolumePressed )
        m_pVolumeTimer->blockSignals( false );

    QToolTip::remove( m_pVolume );
    QToolTip::add( m_pVolume, volumeText()  );
    showMessage( volumeText(), 2000 );
}


/*!
  This will be called whenever the player stops playing.  This can happen
  two ways:

  <ul>
  <li> End of song reached.
  <li> User pushed stop.
  </ul>

  In case the end of song was reached, we want to start loading the next
  song.  We do \e not want to do that if stop is pushed.  This function
  takes care of both these issues.

  \sa stop()
 */

void
QmMainWindow::playingStopped()
{
	if( ! m_StopClicked)
		next();
    else
        setPlayingState( QmMainWindow::Stopped );
}


/*!
  Clears the play list.
*/

void
QmMainWindow::clearPlayList()
{
	stop();
	m_pPlayList->clear();
    updateControls();
    updatePositionControls();
}


/*!
  Removes the current item, if any, from the play list.  If the
  current item is the playing item, it will stop playing and the
  next one, if any, will start playing.
*/

void
QmMainWindow::removeItem()
{
	bool was_playing = m_pPlayer->isPlaying();
	bool stop_playing;
	QmSongItem *newsong = m_pPlayList->removeCurrentItem(&stop_playing);

	if(newsong != 0)
	{
//  		m_pPlayList->setPlaying(newsong);

		// The reason for this variable is to distinguish by user Stop action
		// and end of song Stop.

		m_StopClicked = false;
	
		m_NameString 	  = QString::null;
		m_TimeString 	  = QString::null;
		m_TimeTotalString = QString::null;

		if(was_playing)
			selected(newsong);
	}
	else if(stop_playing)
		stop();

    updateControls();
    updatePositionControls();
}

/*!
  Refreshes the volume slider by reading from the mixer, this is done each 100 ms.
  Adjust the timer value of m_pVolumeTimer to get a better update.
*/

void
QmMainWindow::refreshVolume()
{
    if ( !m_VolumePressed )
    {
        m_pVolume->blockSignals( true );
        int vol = m_Mixer.volume();
        if ( vol != m_pVolume->value() )
        {
            m_pVolume->setValue( vol );
            updateVolume( true, m_ShortMessageTime );
        }
        m_pVolume->blockSignals( false );
    }
}

/*!
  Returns a text string saying "Volume: " and the current volume setting in percentage.
  This is used for tooltips and status bars.
*/

QString
QmMainWindow::volumeText() const
{
    return QString( "Volume: %1%" ).arg( m_pVolume->value() );
}

/*!
  Shows a message in the status bar, this will replace any song info or earlier messages.
  If \a msecs is negative the text is permanent(that is until the song info or another message is show),
  if not this is used as a timer, when the timer times out it will display the song info again.
*/

void
QmMainWindow::showMessage( const QString &msg, int msecs )
{
    if ( msecs > 0 )
    {
        m_pStatusTimer->start( msecs, true );
    }
    m_MessageString = msg;
    m_ShowNameStatus = false;
    updateStatus();
}

/*!
  Resets the status area to show the song info, the is called by the timer on timeout.
  \sa showMessage(), showSongInfo()
*/

void
QmMainWindow::resetStatus()
{
    showSongInfo();
}

/*!
  Displays the song info in the status area, any messages currently shown will be replaced.
  \sa updateStatus()
*/

void
QmMainWindow::showSongInfo()
{
    m_ShowNameStatus = true;
    updateStatus();
}

/*!
  This is called when the volume slider is pressed,
  this makes sure that the volume is not updated when it changes externally.
  ie. The user can change the volume.
  \sa updateVolume()
*/

void
QmMainWindow::volumePressed()
{
    m_pVolumeTimer->blockSignals( true );
    m_VolumePressed = true;
    updateVolume( true );
}

/*!
  This is called when the volume slider is released,
  this makes sure that the volume can be updated when it changes externally.
  \sa updateVolume()
*/

void
QmMainWindow::volumeReleased()
{
    m_pVolumeTimer->blockSignals( false );
    m_VolumePressed = false;
    m_ShowNameStatus = true;
    updateVolume( false );
}

/*!
  Updates the volume information tooltip on the slider.
  If \a in_status is true it will show show the volume information
  in the status bar as well using \a time as the timeout.
*/

void
QmMainWindow::updateVolume( bool in_status, int time )
{
    QToolTip::remove( m_pVolume );
    QToolTip::add( m_pVolume, volumeText() );
    if ( in_status )
        showMessage( volumeText(), time );
    else
        showSongInfo();
}


/*!
  Starts the mpg123 process.
*/

void
QmMainWindow::startMpg123()
{
	if(m_pPlayer == 0)
		m_pPlayer = new QmMpg123Player;
	m_pPlayer->init();
}


/*!
  Sets the current item to be 'bad'.  Bad items will be skipped automatically
  when playing a play list.  Marking items as bad is useful if there are some
  files that the underlaying player cannot cope with, even though the files
  may be correct.

  \sa clearMark()
*/

void
QmMainWindow::setMark()
{
    QListViewItem *item = m_pPlayList->currentItem();
	QmSongItem *song = dynamic_cast<QmSongItem*>(item);

	if(song)
	{
		song->setMark(true);
		updatePositionControls();
	}
}


/*!
  Removes the bad mark of the current item, if any.

  \sa setMark()
*/

void
QmMainWindow::clearMark()
{
    QListViewItem *item = m_pPlayList->currentItem();
	QmSongItem *song = dynamic_cast<QmSongItem*>(item);

	if(song)
	{
		song->setMark(false);
		updatePositionControls();
	}
}


/*!
  Opens the configuration dialog.
*/

void
QmMainWindow::openConfigDialog()
{
    if ( state() == QmMainWindow::Busy ) // Do not show dialogs when busy
        return;

    bool ok;
    QString path = QInputDialog::getText( tr( "Enter absolute path to the mpg123 program" ),
                                          tr( "Path:" ),
                                          QmConfig::instance()->mpg123Path(),
                                          &ok,
                                          this, "config-dialog" );
    if ( ok )
    {
        QmConfig::instance()->setMpg123Path( path );
    }
//	QmConfigDialog dlg;
//	MyDialog2 dlg;
//	dlg.exec();
}

/*! \enum QmMainWindow::MainState
  The state of the main window, tells wheter user interaction is allowed or not
*/

/*! \enum QmMainWindow::MainState QmMainWindow::Ready
  The default state, this means that the user can use the GUI to it's full extent.
*/

/*! \enum QmMainWindow::MainState QmMainWindow::Busy
  Used whenever the program is busy with something and user shouldn't be allowed to use certain parts of the GUI, for instance while a playlist is loading.
*/

/*!
  Returns the main state of the program.
  \sa playingState()
*/

QmMainWindow::MainState
QmMainWindow::state() const
{
    return m_State;
}

/*!
  Sets the main state of the program and enables/disables the various parts of the GUI.
  \sa setPlayingState()
*/

void
QmMainWindow::setState( QmMainWindow::MainState state )
{
    if ( m_State == state )
        return;
    m_State = state;
    updateWidgets();
}

/*! \enum QmMainWindow::PlayState
  The state of the player, used to determine which play controls are enabled/disabled.
*/

/*! \enum QmMainWindow::PlayState QmMainWindow::Stopped
  The default state, no song is playing or one has just been stopped.
*/

/*! \enum QmMainWindow::PlayState QmMainWindow::Playing
  A song is playing.
*/

/*! \enum QmMainWindow::PlayState QmMainWindow::Paused
  A song is playing but is paused.
*/

/*!
  Returns the play state of the program, this tells wheter a song is playing, paused or is stopped.
  \sa state()
*/

QmMainWindow::PlayState
QmMainWindow::playingState() const
{
    return m_PlayState;
}

/*!
  Sets the state of the player, this will update the various parts of the GUI by enabling/disabling buttons and text.
  \sa setState()
*/

void
QmMainWindow::setPlayingState( QmMainWindow::PlayState state )
{
    if ( state == m_PlayState )
        return;
    m_PlayState = state;

    if ( state == QmMainWindow::Stopped )
    {
        m_TimeString = QString::null;
        m_TimeTotalString = QString::null;
    }

    updateStatus();
    updateCaption();
    updateControls();
    updatePositionControls();
}

/*!
  Makes sure the file, playlist and option menus are enabled or disabled. The state of the program decides this.
  \sa state(), setState(), MainState
*/

void
QmMainWindow::updateWidgets()
{
    switch ( m_State )
    {
        case QmMainWindow::Ready:
            m_pAddFiles->setEnabled( true );
            m_pPlaylistButton->setEnabled( true );
            m_pOptions->setEnabled( true );
            break;
        case QmMainWindow::Busy:
            m_pAddFiles->setEnabled( false );
            m_pPlaylistButton->setEnabled( false );
            m_pOptions->setEnabled( false );
            break;
        default:
            break;
            // Should never happen
    }
}

/*!
  Makes sure the play, pause and stop buttons are enabled or disabled according to the playing state. Call this whenever the playlist is modified or the song changes state.
  \sa playingState(), setPlayingState(), PlayState
*/

void
QmMainWindow::updateControls()
{
    switch ( playingState() )
    {
        case QmMainWindow::Playing:
            m_pPlay->setEnabled( m_pPlayList->playingSong() != 0 );
            m_pStop->setEnabled( true );
            m_pPause->setEnabled( true );
            m_pPause->setOn( false );
            break;
        case QmMainWindow::Paused:
            m_pPlay->setEnabled( m_pPlayList->playingSong() != 0 );
            m_pStop->setEnabled( true );
            m_pPause->setEnabled( true );
            m_pPause->setOn( true );
            break;
        case QmMainWindow::Stopped:
            m_pPlay->setEnabled( m_pPlayList->playingSong() != 0 );
            m_pStop->setEnabled( false );
            m_pPause->setEnabled( false );
            m_pPause->setOn( false );
            break;
        default:
            break;
            // Should never happen
    }
}

/*!
  Makes sure the position controls (next and prev) are enabled or disabled according to the song position. Call this whenever the playlist is modified or the current song changes.
*/

void
QmMainWindow::updatePositionControls()
{
    m_pNext->setEnabled( !m_pPlayList->atEnd() );
    m_pPrev->setEnabled( !m_pPlayList->atBeginning() );
}
