/* ;-*-c++-*-
 * qmmainwindow.h
 *
 * $Id: qmmainwindow.h,v 1.31 2000/12/13 18:18:16 mariuss Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef QMMAINWINDOW_H_INCLUDED
#define QMMAINWINDOW_H_INCLUDED

#include <qmainwindow.h>
#include <qfiledialog.h>

#include "qmmenubutton.h"
#include "qmmixer.h"

/*! \file qmmainwindow.h
  Definition of QmMainWindow class which is the main Apollo window.
*/

class QAction;
class QLabel;
class QListViewItem;
class QPopupMenu;
class QSlider;
class QToolButton;
class QVBoxLayout;
class QSize;

class QmPlayList;
class QmMpg123Player;

class QmMainWindow : public QMainWindow
{
    Q_OBJECT
public:
    QmMainWindow(QWidget *parent = 0, const char *name = 0);
    virtual ~QmMainWindow();

    void loadStartupPlayList();
    void saveStartupPlayList();

private slots:
    void addFiles();
	void addDir();
    void loadPlayList();
    void savePlayList();
    void about();
    void aboutQt();
	void openConfigDialog();
    void popupFileMenu();
    void popupPlaylistMenu();
    void popupOptionsMenu();
    void prev();
    void next();
    void play();
    void stop();
	void pause();
    void selected(QListViewItem *item);
    void togglePlaylist();
	void secondsCount(int s);
	void secondsPlaying(int s);
	void setTrackerEnd(int s);
	void startMpg123();
	void setMark();
	void clearMark();
	
	void volumeChanged(int v);
    void volumePressed();
    void volumeReleased();

    void setSongTitle(const QString &title);
	void playingStopped();
	void clearPlayList();
	void removeItem();

    void refreshVolume();
    void resetStatus();

protected:
    void showMessage( const QString &msg, int msecs = -1 );
    void showSongInfo();
    QString volumeText() const;

protected:
    enum MainState
    {
        Ready,
        Busy
    };

    enum PlayState
    {
        Stopped,
        Playing,
        Paused
    };

    MainState state() const;
    PlayState playingState() const;

    void setState( QmMainWindow::MainState state );
    void setPlayingState( QmMainWindow::PlayState state );

private:
    void                updateStatus();
    void                updateCaption();
    void                updateVolume( bool in_status, int time = -1 );
    void                updateWidgets();
    void                updateControls();
    void                updatePositionControls();
	
    QPopupMenu			*m_pFileMenu;
    QPopupMenu			*m_pPlaylistMenu;
    QPopupMenu			*m_pOptionsMenu;
    QmPlayList			*m_pPlayList;
    QWidget             *m_pWidget;
	QAction				*m_pPauseAction;
    QmMenuButton		*m_pAddFiles;
    QmMenuButton		*m_pPlaylistButton;
    QmMenuButton		*m_pOptions;
    QToolButton			*m_pPlay;
    QToolButton			*m_pStop;
    QToolButton			*m_pNext;
    QToolButton			*m_pPrev;
    QToolButton			*m_pPause;
    QToolButton			*m_pRestart;
    QToolButton			*m_pMinimize;
    QLabel				*m_pStatus;
    QVBoxLayout			*top;
	QmMpg123Player 		*m_pPlayer;
	QSlider				*m_pTracker;
	QSlider				*m_pVolume;
    QTimer              *m_pVolumeTimer;
    QTimer              *m_pStatusTimer;

    MainState           m_State;
    PlayState           m_PlayState;

    int                 m_ShortMessageTime;
	bool				m_StopClicked;
    bool                m_ShowNameStatus;
    bool                m_VolumePressed;
    QString             m_MessageString;
    QString             m_NameString;
    QString             m_TimeString;
    QString             m_TimeTotalString;
    QString             m_MusicDir;
    QString             m_PlayListName;
    QFileDialog         m_FileDialog;
	QmMixer				m_Mixer;
};

#endif // QMMAINWINDOW_H_INCLUDED
