/* ;-*-c++-*-
 * qmmpg123player.h
 *
 * $Id: qmmpg123player.h,v 1.16 2000/12/13 17:49:57 amos Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef QMMPG123PLAYER_H_INCLUDED
#define QMMPG123PLAYER_H_INCLUDED

#include "qmplayer.h"

#include <sys/types.h>

/*! \file qmmpg123player.h
  Definition of QmMpg123Player class which provides functionality for communicating with mpg123.
*/

class QSocketNotifier;
class QTimer;

class QmMpg123Player : public QmPlayer
{
	Q_OBJECT
public:
    QmMpg123Player();
    virtual ~QmMpg123Player();

    int init();
	bool isPlaying() const;
	bool isPaused() const;

signals:
	void secondsPlaying(int s);
	void secondsLeft(int s);
	void secondsCount(int s);
	void frameCount(int f);
	void framesLeft(int f);
    void songTitle( const QString &title );
	
public slots:
    virtual void 		play(const QString &Filename);
    virtual void 		play();
    virtual void 		stop();
    virtual void 		pause();
	virtual void		restart();

private slots:
	void dataReceived();
    void releaseTimingInformation();
	
private:
    void parseCodes( char *read_buffer );
	void parseFrameInfo( char *read_buffer );
	void parseInformation( char *read_buffer );
	void parseStatus( char *read_buffer );
	void parsePlayStatus( char *read_buffer );
	void parseError( char *read_buffer );
    QString resolveTags();
	bool sendCommand( const char *write_buffer );
    void startTimer();

	static void childHandler(int sig);
	static void failHandler(int sig);
	
    int fd_send[2];
    int fd_recv[2];

	static const int READ_BUF_SIZE = 1024;
	static const int WRITE_BUF_SIZE = 1024;
	char m_ReadBuffer[READ_BUF_SIZE+1];
	char m_WriteBuffer[WRITE_BUF_SIZE];

	QSocketNotifier *m_pReadNotifier;
    QTimer          *m_pTimer;

    QString m_ID3Format;
    QString m_Filename;
    QString m_FilePath;
    QString m_FileExtension;
    QString m_Artist;
    QString m_Album;
    QString m_Title;
    QString m_Year;
    QString m_Comment;
    QString m_Genre;

//  	int m_OldTime;
	bool 		m_Playing;
	bool 		m_Paused;
    bool 		m_NewSong;
    bool 		m_GotNewSong;
    int         m_BufferLeftovers;
	int 		m_SendFd;  		// send commands to mpg123
	int 		m_RecvFd;
	int			m_Pid;
	static bool	s_OurTermSignal;
    int 		m_FrameCount;
    int 		m_Seconds;
	fd_set 		mpg123_fdset;
};

#endif // QMMPG123PLAYER_H_INCLUDED
