/* qmplaylistitem.cpp
 *
 * $Id: qmplaylistitem.cpp,v 1.1 2000/12/10 11:35:25 amos Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#include "qmplaylistitem.h"

#include <qstring.h>

#include "qmplaylist.h"

/*! \file qmplaylistitem.cpp
  Implementation of QmPlayListItem class which represents an item in the play list.
*/

/*!
  \class QmPlayListItem qmplaylistitem.h
  \brief Represents an item in the play list.

  This is the base class for items in the play list.  It provides
  a common interface for persistant storage.
*/

/*!
  Listview constructor
*/

QmPlayListItem::QmPlayListItem(QListView *parent, int count)
    : QListViewItem(parent),
    m_Count( count >= 0 ? count : QmPlayList::count() )
{
}

/*!
  Listview item constructor
*/

QmPlayListItem::QmPlayListItem(QmPlayListItem *parent, int count)
    : QListViewItem(parent),
    m_Count( count >= 0 ? count : QmPlayList::count() )
{
}

/*!
  Destroys the object
*/

QmPlayListItem::~QmPlayListItem()
{
}

/*!
  Returns the internal counter for this item.
*/

int QmPlayListItem::sortCounter() const
{
    return m_Count;
}

/*!
  Returns a string of numbers by using the sortCounter(),
  this string is used for sorting playlist items properly.
*/

QString QmPlayListItem::sortCounterString() const
{
    QString tmp;
    tmp.sprintf( "%.10d", sortCounter() );
    return tmp;
}
