/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef __MIXMAGIC_BLOCK_H__
#define __MIXMAGIC_BLOCK_H__

#include <tree.h>
#include <parser.h>
#include <gtk/gtk.h>
#include "mainwin.h"


#define BLOCK_HEIGHT 100
#define BLOCK_HANDLE_SIZE 5
#define MIN_BLOCK_HEIGHT 10
#define BLOCK_MIN_WIDTH (8 * block->track->song->x_scale)

#ifndef TRACK_DEFINED
typedef struct _Track Track;
#define TRACK_DEFINED
#endif

typedef enum _EnvelopeMode EnvelopeMode;

enum _EnvelopeMode {
        PAN_ENVELOPE,
        AMP_ENVELOPE,
	NO_ENVELOPE
};

#ifndef BLOCK_DEFINED
typedef struct _Block Block;
#endif
typedef struct _BlockClass  BlockClass;
typedef struct _Envelope Envelope;

#define BLOCK(obj)         GTK_CHECK_CAST(obj, block_get_type (), Block)
#define BLOCK_CLASS(klass)  GTK_CHECK_CLASS_CAST(klass, block_get_type (), BlockClass)
#define IS_BLOCK(obj)       GTK_CHECK_TYPE(obj, block_get_type ())

struct _Block {
        GtkDrawingArea drawingarea;	/* Parent object */
	GtkWidget *popup_menu;
	GdkPixmap *pixmap;
	Mainwin *mainwin;

	GdkGC *wav_gc;			/* GC used to draw the wave form */
	GdkGC *amp_gc;			/* GC used to draw the amplitude envelope */
	GdkGC *pan_gc;			/* GC used to draw the pan envelope */
	GdkGC *selection_gc;		/* GC used to draw the selection */

	gint selection_flag;		/* True if we should draw the selection */
	gint selection_start;		/* First selection position, in samples (not pixels)*/
	gint selection_end;		/* Last selection position, in samples (not pixels) */

	gint old_width;
	gint old_height;

	gboolean click_flag;		/* True if the left button is pressed */
	GdkPoint click_point;		/* Local point where the left button was pressed */

	Track *track;			/* A pointer to the track this block is in */

	GList *closest_point;		/* A pointer to the active node of the active envelope or NULL */
	EnvelopeMode envelope_mode;	/* Holds information of the current envelope type is */

	GdkCursorType cursor_type;	/* The active cursor type */

	/* Old block */
	gint start;			/* Offset from the beginning of the sample */
	gint length;			/* The length of this block */
	gint position;			/* Position in time */

	Sample *sample;			/* The sample assosiated with this block */

	GList *amp_env;			/* Pointer to this blocks amplitude envelope or NULL */
	GList *pan_env;			/* Pointer to this blocks pan envelope or NULL */

};

struct _BlockClass 
{
        GtkDrawingAreaClass parent_class;
	void (* mix_into_buffer) (Block *block, gint16 *buffer, gint position, gint length);
	void (* lower_left_box_motion) (Block *block, GdkEventMotion *event, gint x, gint y);
	void (* lower_right_box_motion) (Block *block, GdkEventMotion *event, gint x, gint y);
	void (* lower_half_motion) (Block *block, GdkEventMotion *event, gint x, gint y);
	void (* upper_half_motion) (Block *block, GdkEventMotion *event, gint x, gint y);
};

struct _Envelope {
	gint x;
	gfloat y;
};

guint		block_get_type	(void);
GtkWidget*	block_new	(Track *track);
void		block_update	(Block *block);
void		block_mix_into_buffer (Block *block, gint16 *buffer, gint position, gint length);
void		block_load (Song *song, Track *track, xmlNodePtr node, xmlNsPtr ns, gint track_num);
#endif
