/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#include <gnome.h>
#include "mixmagic.h"
#include "mixer.h"
#include "mainwin.h"
#include "track.h"
#include "block.h"
#include "sample.h"
#include "song.h"
#include "stock.h"
#include "preferences.h"

static gint mainwin_session_count = 0;
static GdkAtom block_atom = GDK_NONE;
static GtkObjectClass *parent_class = NULL;

void about_show ();
static void mainwin_destroy (GtkObject *object);
static void mainwin_class_init (MainwinClass *class);
static void mainwin_init (Mainwin *mainwin);
static void mainwin_zoom_in_clicked (GtkWidget *widget, Mainwin *mainwin);
static void mainwin_zoom_out_clicked (GtkWidget *widget, Mainwin *mainwin);
static void mainwin_zoom_fit_clicked (GtkWidget *widget, Mainwin *mainwin);
static void mainwin_close_clicked (GtkWidget *widget, gpointer data);
static void mainwin_exit_clicked (GtkWidget *widget, gpointer data);
static gboolean mainwin_delete_event (GtkWidget *widget, gpointer data);
static void mainwin_save_clicked (GtkWidget *widget, gpointer data);
static void mainwin_save_as_clicked (GtkWidget *widget, gpointer data);
static void mainwin_new_clicked (GtkWidget *widget, gpointer data);
static void mainwin_preferences_clicked (GtkWidget *widget, gpointer data);
static void mainwin_open_clicked (GtkWidget *widget, gpointer data);
static void mainwin_about_clicked (GtkWidget *widget, gpointer data);
static void mainwin_repeat_toggled (GtkWidget *widget, Mainwin *mainwin);
static void mainwin_selection_received (GtkWidget *widget, GtkSelectionData *selection_data, guint time, gpointer data);
static void mainwin_rewind_clicked (GtkWidget *widget, Mainwin *mainwin);

/*
 * Mainwin menus 
 */

/* Don't forget to update "file_menu_index" if you change the menu structure */

static GnomeUIInfo file_menu [] = {
	GNOMEUIINFO_MENU_NEW_ITEM ("New song", "Create a new song.", mainwin_new_clicked, NULL),
	GNOMEUIINFO_MENU_OPEN_ITEM (mainwin_open_clicked, NULL),
	GNOMEUIINFO_MENU_SAVE_ITEM (mainwin_save_clicked, NULL),
	GNOMEUIINFO_MENU_SAVE_AS_ITEM (mainwin_save_as_clicked, NULL),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_CLOSE_ITEM (mainwin_close_clicked, NULL),
	GNOMEUIINFO_MENU_EXIT_ITEM (mainwin_exit_clicked, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo view_menu[] = {
	GNOMEUIINFO_ITEM_NONE ("Zoom in", "Zoom in", mainwin_zoom_in_clicked),
	GNOMEUIINFO_ITEM_NONE ("Zoom out", "Zoom out", mainwin_zoom_out_clicked),
	GNOMEUIINFO_ITEM_NONE ("Zoom fit", "Zoom fit", mainwin_zoom_fit_clicked),
	GNOMEUIINFO_END
};

static GnomeUIInfo song_menu[] = {
	GNOMEUIINFO_ITEM_NONE ("Play", "Play song", mainwin_play_clicked),
	GNOMEUIINFO_ITEM_NONE ("Stop", "Stop song", mainwin_stop_clicked),
	GNOMEUIINFO_END
};

static GnomeUIInfo settings_menu[] = {
	GNOMEUIINFO_MENU_PREFERENCES_ITEM (mainwin_preferences_clicked, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo help_menu [] = {
	GNOMEUIINFO_MENU_ABOUT_ITEM (mainwin_about_clicked, NULL),
	GNOMEUIINFO_END
};

static GnomeUIInfo main_menu[] = {
	GNOMEUIINFO_MENU_FILE_TREE (file_menu),
	GNOMEUIINFO_MENU_VIEW_TREE (view_menu),
	GNOMEUIINFO_SUBTREE ("_Song", song_menu),
	GNOMEUIINFO_MENU_SETTINGS_TREE (settings_menu),
	GNOMEUIINFO_MENU_HELP_TREE (help_menu),
	GNOMEUIINFO_END
};

#define TOOLBAR_PLAY_INDEX 6
static GnomeUIInfo mixmagic_toolbar[] = {
	GNOMEUIINFO_ITEM_STOCK ("New", "Creates a new song.",
				mainwin_new_clicked, GNOME_STOCK_PIXMAP_NEW),
	GNOMEUIINFO_ITEM_STOCK ("Open", "Open an existing song.",
				mainwin_open_clicked, GNOME_STOCK_PIXMAP_OPEN),
	GNOMEUIINFO_ITEM_STOCK ("Save", "Save the song to a file.",
				mainwin_save_clicked, GNOME_STOCK_PIXMAP_SAVE),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK ("Rewind", "Rewind to the beginning.",
				mainwin_rewind_clicked, STOCK_REWIND),
	GNOMEUIINFO_ITEM_STOCK ("Stop", "Stop the playback.",
				mainwin_stop_clicked, STOCK_STOP),
	GNOMEUIINFO_ITEM_STOCK ("Play", "Start playback.",
				mainwin_play_clicked, STOCK_PLAY),
	{GNOME_APP_UI_TOGGLEITEM, "Repeat", "Restart at the beginning, when the player gets to the end of the song.",
	 mainwin_repeat_toggled, NULL, NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_PIXMAP_REFRESH },
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK ("Zoom in", "Zoom in.",
				mainwin_zoom_in_clicked, STOCK_ZOOM_IN),
	GNOMEUIINFO_ITEM_STOCK ("Zoom out", "Zoom out.",
				mainwin_zoom_out_clicked, STOCK_ZOOM_OUT),
	GNOMEUIINFO_ITEM_STOCK ("Zoom fit", "Zoom fit.",
				mainwin_zoom_fit_clicked, STOCK_ZOOM_FIT),
	GNOMEUIINFO_END
};

guint
mainwin_get_type ()
{
        static guint mainwin_type = 0;
	
        if (!mainwin_type) {

		GtkTypeInfo mainwin_info = {
			"Mainwin",
			sizeof (Mainwin),
			sizeof (MainwinClass),
			(GtkClassInitFunc) mainwin_class_init,
			(GtkObjectInitFunc) mainwin_init,
			(GtkArgSetFunc) NULL,
			(GtkArgGetFunc) NULL
		};
	    
		mainwin_type = gtk_type_unique (gtk_object_get_type (), &mainwin_info);
          }
	
        return mainwin_type;
}

static void
mainwin_class_init (MainwinClass *class)
{
	GtkObjectClass *object_class;

	parent_class = gtk_type_class (GTK_TYPE_OBJECT);

	object_class = (GtkObjectClass*) class;
	object_class->destroy = mainwin_destroy;
}

static void
mainwin_init (Mainwin *mainwin)
{
}

static void 
mainwin_destroy (GtkObject *object)
{
	Mainwin *mainwin = MAINWIN (object);

	g_print ("mainwin_destroy\n");

	if (mainwin->filename)
		g_free (mainwin->filename);

	gtk_widget_destroy (mainwin->toplevel);

	if (mainwin->open_file_sel)
		gtk_widget_destroy (mainwin->open_file_sel);

	if (mainwin->save_file_sel)
		gtk_widget_destroy (mainwin->save_file_sel);

	(*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

/*
 * Adds a toolbar to the mainwin 
 */
static void
mainwin_create_toolbars (Mainwin *mainwin)
{
	GtkAccelGroup *accel_group;

	gnome_app_create_toolbar_with_data (GNOME_APP (mainwin->toplevel), mixmagic_toolbar, mainwin);
	
	accel_group = gtk_accel_group_new ();
       	gtk_accel_group_attach (accel_group, GTK_OBJECT (mainwin->toplevel));
	
	gtk_widget_add_accelerator (mixmagic_toolbar[TOOLBAR_PLAY_INDEX].widget,
				    "clicked",
				    accel_group,
				    GDK_space,
				    0,
				    GTK_ACCEL_VISIBLE | GTK_ACCEL_SIGNAL_VISIBLE);

}

/*
 * Creates a window 
*/
GtkObject *
mainwin_new (const gchar *filename)
{
	Mainwin *mainwin;
	gchar *title;

	mainwin_session_count++;

	/* Allocate new structure */
	mainwin = MAINWIN (gtk_type_new (mainwin_get_type ()));

	mainwin->toplevel = gnome_app_new ("mixmagic", "MixMagic");
	
	gtk_signal_connect (GTK_OBJECT (mainwin->toplevel), "delete_event",
			    GTK_SIGNAL_FUNC (mainwin_delete_event), mainwin);

	gtk_signal_connect (GTK_OBJECT (mainwin->toplevel), "selection_received",
			    GTK_SIGNAL_FUNC (mainwin_selection_received), mainwin);


	gnome_app_create_menus_with_data (GNOME_APP (mainwin->toplevel), main_menu, mainwin);

	mainwin_create_toolbars (mainwin);

	mainwin->appbar = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_USER);
	gnome_app_set_statusbar (GNOME_APP (mainwin->toplevel), mainwin->appbar);

	/* Create a new song class */
	mainwin->song = SONG (song_new (GNOME_APPBAR (mainwin->appbar)));

	/* Load a file if a filename is specified */
	if (filename != NULL) {
		song_load (mainwin->song, filename);

		mainwin->filename = g_strdup (filename);
		title = g_strdup_printf ("MixMagic: %s", g_basename (filename));
		gtk_window_set_title (GTK_WINDOW (mainwin->toplevel), title);
		g_free (title);
#if 0
		song_zoom_fit (mainwin->song);
#endif
	}

	gnome_app_set_contents (GNOME_APP (mainwin->toplevel), GTK_WIDGET (mainwin->song));
	gtk_widget_show (GTK_WIDGET (mainwin->song));

	gtk_window_set_default_size (GTK_WINDOW (mainwin->toplevel), 500, 400);

	gtk_widget_show_all (mainwin->toplevel);

	return GTK_OBJECT (mainwin);
}

/*
 * Close the window.
 */
static gboolean
mainwin_delete_event (GtkWidget *widget, gpointer data)
{
	mainwin_session_count--;

	/* 
	 *  Terminate the app if this is the last instance 
	 */
	if (mainwin_session_count == 0)
		gtk_main_quit ();
	else
		gtk_object_destroy (GTK_OBJECT (data));

	return FALSE;
}

/*
 * Zoom In.
 */
static void
mainwin_zoom_in_clicked (GtkWidget *widget, Mainwin *mainwin)
{
	song_zoom_in (mainwin->song);
}

/*
 * Zoom out.
 */
static void
mainwin_zoom_out_clicked (GtkWidget *widget, Mainwin *mainwin)
{
	song_zoom_out (mainwin->song);
}

/*
 * Zoom fit
 */
static void
mainwin_zoom_fit_clicked (GtkWidget *widget, Mainwin *mainwin)
{
	song_zoom_fit (mainwin->song);
}

/*
 * Close the window.
 */
static void
mainwin_close_clicked (GtkWidget *widget, gpointer data)
{
	mainwin_session_count--;

	/* Terminate the app if this is the last instance */
	if (mainwin_session_count == 0)
		mainwin_exit_clicked (NULL, data);
	else
		gtk_object_destroy (GTK_OBJECT (data));
}

/*
 * Exit the app.
 */
static void
mainwin_exit_clicked (GtkWidget *widget, gpointer data)
{
	Mainwin *mainwin = MAINWIN (data);

	song_stop_playback (mainwin->song);
	gtk_main_quit ();
}

/*
 * User clicked 'New'.
 */
static void
mainwin_new_clicked (GtkWidget *widget, gpointer data)
{
	mainwin_new (NULL);
}

/*
 * User clicked 'Preferences'.
 */
static void
mainwin_preferences_clicked (GtkWidget *widget, gpointer data)
{
	Mainwin *mainwin = MAINWIN (data);

	/* Stop the playback to avoid segfaults when changing output plugins */
	song_stop_playback (mainwin->song);

	preferences_show ();
}

/*
 * Called when the use clicks on the "cancel" button in a GtkFileselector.
 */
static void
mainwin_file_sel_cancel_clicked (GtkWidget *widget, gpointer data)
{
	GtkWidget *file_sel = (GtkWidget *)data;

	g_assert (file_sel != NULL);
	
	gtk_widget_hide (file_sel);
}

/*
 * Called when the use clicks on the "ok" button.
 */
static void
mainwin_open_ok_clicked (GtkWidget *widget, gpointer data)
{
	Mainwin *mainwin = (Mainwin *)data;
	Mainwin *new_win;
	gchar *filename, *title;

	g_assert (mainwin != NULL);

	filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (mainwin->open_file_sel)));

	if (song_is_empty (mainwin->song)) {

		if (mainwin->filename)
			g_free (mainwin->filename);

		mainwin->filename = filename;
		
		if (song_load (mainwin->song, filename)) {

			title = g_strdup_printf ("MixMagic: %s", g_basename (filename));
			gtk_window_set_title (GTK_WINDOW (mainwin->toplevel), title);
			g_free (title);

			song_zoom_fit (mainwin->song);
		}
	} else {

		new_win = MAINWIN (mainwin_new (filename));

		g_free (filename);
	}

	/* Hide the file selection dialog */
	gtk_widget_hide (mainwin->open_file_sel);
}

/*
 * Called when the use clicks on the "ok" button.
 */
static void
mainwin_save_ok_clicked (GtkWidget *widget, gpointer data)
{
	Mainwin *mainwin = (Mainwin *)data;
	gchar *title;

	g_assert (mainwin != NULL);

	if (mainwin->filename)
		g_free (mainwin->filename);

	mainwin->filename = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (mainwin->save_file_sel)));

	if (song_save (mainwin->song, mainwin->filename)) {

		title = g_strdup_printf ("MixMagic: %s", g_basename (mainwin->filename));
		gtk_window_set_title (GTK_WINDOW (mainwin->toplevel), title);
		g_free (title);
	}
	/* Hide the file selection dialog */
	gtk_widget_hide (mainwin->save_file_sel);
}

/*
 * User clicked 'About'.
 */
static void
mainwin_about_clicked (GtkWidget *widget, gpointer data)
{
	about_show ();
}

/*
 * User clicked 'Open'.
 */
static void
mainwin_open_clicked (GtkWidget *widget, gpointer data)
{
	Mainwin *mainwin = (Mainwin *)data;

	g_assert (mainwin != NULL);
	
	if (mainwin->open_file_sel == NULL) {

		mainwin->open_file_sel = gtk_file_selection_new ("Open song (*.mmsong)");
		
		gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (mainwin->open_file_sel)->ok_button), "clicked",
			    GTK_SIGNAL_FUNC (mainwin_open_ok_clicked), mainwin);

		gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (mainwin->open_file_sel)->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC (mainwin_file_sel_cancel_clicked), mainwin->open_file_sel);

	}
	gtk_widget_show (mainwin->open_file_sel);
}

/*
 * User clicked 'save'.
 */
static void
mainwin_save_clicked (GtkWidget *widget, gpointer data)
{
	Mainwin *mainwin = (Mainwin *)data;
	
	if (mainwin->filename)
		song_save (mainwin->song, mainwin->filename);
	else {
		if (mainwin->save_file_sel == NULL) {
			
			mainwin->save_file_sel = gtk_file_selection_new ("Save song (*.mmsong)");
			
			gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (mainwin->save_file_sel)->ok_button), "clicked",
					    GTK_SIGNAL_FUNC (mainwin_save_ok_clicked), mainwin);
			
			gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (mainwin->save_file_sel)->cancel_button), "clicked",
					    GTK_SIGNAL_FUNC (mainwin_file_sel_cancel_clicked), mainwin->save_file_sel);
			
		}
		gtk_widget_show (mainwin->save_file_sel);
	}
}

/*
 * User clicked 'save'.
 */
static void
mainwin_save_as_clicked (GtkWidget *widget, gpointer data)
{
	Mainwin *mainwin = (Mainwin *)data;

	if (mainwin->save_file_sel == NULL) {
		
		mainwin->save_file_sel = gtk_file_selection_new ("Save song (*.mmsong)");
		
		gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (mainwin->save_file_sel)->ok_button), "clicked",
				    GTK_SIGNAL_FUNC (mainwin_save_ok_clicked), mainwin);
		
		gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (mainwin->save_file_sel)->cancel_button), "clicked",
				    GTK_SIGNAL_FUNC (mainwin_file_sel_cancel_clicked), mainwin->save_file_sel);
		
	}
	gtk_widget_show (mainwin->save_file_sel);
}

/*
 * Signal handler called when the selections owner returns the data 
 */
static void 
mainwin_selection_received (GtkWidget *widget, GtkSelectionData *selection_data, guint time, gpointer data)
{
	Block *block;
	Mainwin *mainwin = MAINWIN (data);

	/* Is it a MMBlock selection */
	if (selection_data->type != block_atom) {

		/* Start playback from the beginning */
		song_play (mainwin->song);
		return;
	}

	block = *(Block **)selection_data->data;

	/*
	 * Block selected, just play the selection.
	 * The block gets deleted by the mixer.
	 */
	
	song_play_block_and_delete (mainwin->song, block);	
}

/*
 * User clicked 'Rewind'.
 */
static void
mainwin_rewind_clicked (GtkWidget *widget, Mainwin *mainwin)
{
	song_stop_playback (mainwin->song);
	song_move_marker (mainwin->song, 0);

	gtk_adjustment_set_value (song_get_adjustment (mainwin->song), 0);
}

/*
 * User clicked 'play'.
 */
void
mainwin_play_clicked (GtkWidget *widget, Mainwin *mainwin)
{
        /* Get the atom corresponding to the string "Block" */
        if (block_atom == GDK_NONE)
		block_atom = gdk_atom_intern ("Block", FALSE);

	/* If we are allready playing, then stop */
	if (mixer_is_playing ())
		song_stop_playback (mainwin->song);
	else {
		/* Else Check if a block is selected */

		/* Tell mainwin_selection_received to play the selection */
		gtk_selection_convert (mainwin->toplevel, GDK_SELECTION_PRIMARY, 
				       block_atom,
				       GDK_CURRENT_TIME);
	}
}

/*
 * User clicked 'stop'.
 */
void
mainwin_stop_clicked (GtkWidget *widget, Mainwin *mainwin)
{
	song_stop_playback (mainwin->song);
}

/*
 * User toggled 'Repeat'.
 */
static void
mainwin_repeat_toggled (GtkWidget *widget, Mainwin *mainwin)
{
	mainwin->song->repeat_flag = GTK_TOGGLE_BUTTON (widget)->active;
}
