/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef __MIXMAGIC_SONG_H__
#define __MIXMAGIC_SONG_H__

#include <gnome.h>
#include "sample.h"

#define SONG(obj)         GTK_CHECK_CAST(obj, song_get_type (), Song)
#define SONG_CLASS(klass)  GTK_CHECK_CLASS_CAST(klass, song_get_type (), SongClass)
#define IS_SONG(obj)       GTK_CHECK_TYPE(obj, song_get_type ())

#define BLOCK_DEFINED
typedef struct _Block Block;
typedef struct _Song Song;
typedef struct _SongClass SongClass;

struct _Song {
	GtkVBox box;

	GHashTable *sample_hash;
	GList *track_list;

	gint x_scale;
	gint block_height;

	GtkWidget *scrollbar;
	GnomeAppBar *appbar;

	guint marker_id;

	gint marker_position;

	gboolean repeat_flag;

	gboolean going_unsolo_flag; /* This flag is used to avoid infinite loops when "Solo" is toggled */
};

struct _SongClass 
{
        GtkVBoxClass parent_class;
};

GtkWidget *	song_new		(GnomeAppBar *appbar);
guint		song_get_type		(void);
gboolean 	song_is_empty		(Song *song);
gboolean 	song_load 		(Song *song, const gchar *filename);
gboolean 	song_save 		(Song *song, const gchar *filename);
void 		song_move_block		(Song *song, Block *block, gint track_num);
void 		song_insert_block 	(Song *song, Block *block, gint track_num);
void 		song_delete_block	(Song *song, Block *block);
void		song_dump_info		(Song *song);
Sample *	song_get_sample		(Song *song, gchar *filename);
GtkAdjustment*	song_get_adjustment	(Song *song);
void		song_set_adjustment	(Song *song, GtkAdjustment *adjustment);
void		song_update		(Song *song);
void		song_play		(Song *song);
void		song_play_block_and_delete (Song *song, Block *block);
void		song_stop_playback	(Song *song);
void		song_update_scrollbar 	(Song *song);
gint		song_get_length 	(Song *song); 
void		song_zoom_in 		(Song *song); 
void		song_zoom_out 		(Song *song); 
void		song_zoom_fit 		(Song *song); 
void		song_move_marker	(Song *song, gint position);
#endif
